"""
This package contains the pieces to run the Engine

* Engine configuration (FlaskApp, Store, Modules...)
* CMEManager, used to communicate the Engine with the API and the Store
* Scheduler to schedule executors for later use
"""
